"""
Upsonic Tools Module
This module contains the tool implementations that can be used with Upsonic.
"""
from typing import Any, List, Dict, Optional, Type, Union, Callable

class Search:
    pass

class ComputerUse:
    pass

class BrowserUse:
    pass

# Export all tool classes
__all__ = ["Search", "ComputerUse", "BrowserUse"] 